package com.zsc.edu.gateway.framework.json;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.springframework.stereotype.Component;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zhuang
 */
@Component
public class JsonTypeHandler extends BaseTypeHandler<Object> {

    private static final ObjectMapper objectMapper = new ObjectMapper();

    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, Object parameter, JdbcType jdbcType) throws SQLException {
        try {
            String jsonString;
            if (parameter instanceof Map) {
                jsonString = objectMapper.writeValueAsString(parameter);
            } else if (parameter instanceof String) {
                jsonString = (String) parameter;
            } else if (parameter instanceof List) {
                jsonString = objectMapper.writeValueAsString(parameter);
            } else {
                throw new SQLException("Unsupported parameter type: " + parameter.getClass().getName());
            }
            ps.setString(i, jsonString);
        } catch (JsonProcessingException e) {
            throw new SQLException("Error converting object to JSON", e);
        }
    }

    @Override
    public Object getNullableResult(ResultSet rs, String columnName) throws SQLException {
        String json = rs.getString(columnName);
        return parseJson(json);
    }

    @Override
    public Object getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        String json = rs.getString(columnIndex);
        return parseJson(json);
    }

    @Override
    public Object getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        String json = cs.getString(columnIndex);
        return parseJson(json);
    }

    private Object parseJson(String json) {
        if (json == null) {
            return null;
        }
        try {
            if (json.startsWith("{")) {
                return objectMapper.readValue(json, HashMap.class);
            } else if (json.startsWith("[")) {
                return objectMapper.readValue(json, List.class);
            } else {
                throw new RuntimeException("Invalid JSON format");
            }
        } catch (Exception e) {
            throw new RuntimeException("Error parsing JSON", e);
        }
    }
}
